# -*- coding: utf-8 -*-
##############################################################################
#
#    Remembership module for OpenERP, Overload membership module
#    Copyright (C) 2012 L'Heureux Cyclage (<http://www.heureux-cyclage.org>) Ludovic CHEVALIER
#
#    This file is a part of Remembership
#
#    Remembership is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Remembership is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
import time
from datetime import datetime, date
from dateutil.relativedelta import relativedelta


class Partner(osv.osv):
    _inherit = 'res.partner'

    # XXX: copied from openerp/addons/membership/membership.py
    def _membership_date(self, cr, uid, ids, name, args, context=None):
        """Return  date of membership"""
        name = name[0]
        res = {}
        member_line_obj = self.pool.get('membership.membership_line')
        for partner in self.browse(cr, uid, ids, context=context):
            # XXX: commented part from openerp/addons/membership/membership.py
            #if partner.associate_member:
            #     partner_id = partner.associate_member.id
            #else:
            # NOTE: hmm, ça devrait être toujours partner.id
            partner_id = partner.id
            res[partner.id] = {
                 'membership_start': False,
                 'membership_stop': False,
                 'membership_cancel': False
            }
            if name == 'membership_start':
                line_id = member_line_obj.search(cr, uid, [('partner', '=', partner_id),('date_cancel','=',False)],
                            limit=1, order='date_from', context=context)
                if line_id:
                    res[partner.id]['membership_start'] = member_line_obj.read(cr, uid, line_id[0],
                            ['date_from'], context=context)['date_from']

            if name == 'membership_stop':
                line_id1 = member_line_obj.search(cr, uid, [('partner', '=', partner_id),('date_cancel','=',False)],
                            limit=1, order='date_to desc', context=context)
                if line_id1:
                    res[partner.id]['membership_stop'] = member_line_obj.read(cr, uid, line_id1[0],
                              ['date_to'], context=context)['date_to']

            if name == 'membership_cancel':
                if partner.membership_state == 'canceled':
                    line_id2 = member_line_obj.search(cr, uid, [('partner', '=', partner.id)], limit=1, order='date_cancel', context=context)
                    if line_id2:
                        res[partner.id]['membership_cancel'] = member_line_obj.read(cr, uid, line_id2[0], ['date_cancel'], context=context)['date_cancel']
        return res
    # XXX: copied from openerp/addons/membership/membership.py
    def _get_partner_id(self, cr, uid, ids, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res_obj =  self.pool.get('res.partner')
        data_inv = member_line_obj.browse(cr, uid, ids, context=context)
        list_partner = []
        for data in data_inv:
            list_partner.append(data.partner.id)
        ids2 = list_partner
        while ids2:
            ids2 = res_obj.search(cr, uid, [('associate_member', 'in', ids2)], context=context)
            list_partner += ids2
        return list_partner
    # XXX: copied from openerp/addons/membership/membership.py
    def _get_invoice_partner(self, cr, uid, ids, context=None):
        inv_obj = self.pool.get('account.invoice')
        res_obj = self.pool.get('res.partner')
        data_inv = inv_obj.browse(cr, uid, ids, context=context)
        list_partner = []
        for data in data_inv:
            list_partner.append(data.partner_id.id)
        ids2 = list_partner
        while ids2:
            ids2 = res_obj.search(cr, uid, [('associate_member', 'in', ids2)], context=context)
            list_partner += ids2
        return list_partner

    _columns = {
        'member_ident': fields.char('Member identifier', size=64),
        'associate_members': fields.one2many('res.partner', 'associate_member', 'Associate members', help='Members who are associated to this partner.'),
        # XXX: copied from openerp/addons/membership/membership.py
        'membership_start': fields.function(
                    _membership_date, multi = 'membership_start',
                    string = 'Membership Start Date', type = 'date',
                    store = {
                        'account.invoice': (_get_invoice_partner, ['state'], 10),
                        'membership.membership_line': (_get_partner_id, ['state'], 10, ),
                        'res.partner': (lambda self, cr, uid, ids, c={}: ids, ['free_member'], 10)
                    }, help="Date from which membership becomes active."),
        # XXX: copied from openerp/addons/membership/membership.py
        'membership_stop': fields.function(
                    _membership_date,
                    string = 'Membership End Date', type='date', multi='membership_stop',
                    store = {
                        'account.invoice': (_get_invoice_partner, ['state'], 10),
                        'membership.membership_line': (_get_partner_id, ['state'], 10),
                        'res.partner': (lambda self, cr, uid, ids, c={}: ids, ['free_member'], 10)
                    }, help="Date until which membership remains active."),
        # XXX: copied from openerp/addons/membership/membership.py
        'membership_cancel': fields.function(
                    _membership_date,
                    string = 'Cancel Membership Date', type='date', multi='membership_cancel',
                    store = {
                        'account.invoice': (_get_invoice_partner, ['state'], 11),
                        'membership.membership_line': (_get_partner_id, ['state'], 10),
                        'res.partner': (lambda self, cr, uid, ids, c={}: ids, ['free_member'], 10)
                    }, help="Date on which membership has been cancelled"),
    }

    _sql_constraints = [
        ('member_ident_uniq', 'unique(member_ident, company_id)', 'The member identifier must be unique !'),
    ]

    def create_membership_invoice(self, cr, uid, ids, product_id=None, datas=None, context=None):
        # NOTE: use context to supply date_from to account_invoice_line.create
        ctx = context and context.copy() or {}
        ctx['date_from'] = datas.get('date_from', None)
        return super(Partner, self).create_membership_invoice(cr, uid, ids, product_id=product_id, datas=datas, context=ctx)

Partner()


class Product(osv.osv):
    _inherit = 'product.product'

    _columns = {
        'membership_grouped': fields.boolean('Grouped membership product', help='Check if it\'s a grouped membership product.'),
        'membership_date2date': fields.boolean('Date to date membership product', required=False, help='Check if it\'s a date to date membership product.'),
    }

Product()


class account_invoice_line(osv.osv):
    _inherit = 'account.invoice.line'

    def write(self, cr, uid, ids, vals, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res = super(account_invoice_line, self).write(cr, uid, ids, vals, context=context)
        for line in self.browse(cr, uid, ids, context=context):
            if line.invoice_id.type == 'out_invoice':
                ml_ids = member_line_obj.search(cr, uid, [('account_invoice_line', '=', line.id)], context=context)
                if line.product_id and line.product_id.membership:
                    member_line_id = member_line_obj.search(cr, uid
                      , [('partner'             , '=', line.invoice_id.partner_id.id)
                        ,('account_invoice_line', '=', line.id)]
                      , limit=1
                      , context=context)
                    for member_line in member_line_obj.browse(cr, uid, member_line_id, context=context):
                      # NOTE: get member_line created in membership.account_invoice_line.create
                        date_from = member_line.date_from
                        date_to   = member_line.date_to
                        if line.product_id.membership_date2date:
                            date_from = context['date_from'] and datetime.strptime(context['date_from'], "%Y-%m-%d") or date.today()
                            date_to   = date_from + relativedelta(months = +12) # TODO: parameterize this delta?
                            date_from = date_from.strftime("%Y-%m-%d")
                            date_to   = date_to  .strftime("%Y-%m-%d")
                        member_line_obj.write(cr, uid, member_line.id
                          , {'date_from': date_from
                            ,'date_to'  : date_to
                            }
                          , context=context)
                    if line.product_id.membership_grouped:
                        if line.invoice_id.partner_id.associate_members:
                            associate_member_line_ids = member_line_obj.search(cr, uid
                              , [ ('account_invoice_line', '=', line.id)
                                , ('partner', '!=', line.invoice_id.partner_id.id)
                                ]
                              , context=context)
                            for associate_member_line in member_line_obj.browse(cr, uid, associate_member_line_ids, context=context):
                                member_line_obj.write(cr, uid, associate_member_line.id
                                  , {'date_from': date_from
                                    ,'date_to'  : date_to
                                    }
                                  , context=context)
                        else:
                            print("DEV: mettre une contrainte pour l'objet\
                                  membership.membership_line interdisant les\
                                  adhésions groupées reliées à des partenaires\
                                  sans membres associés")
                    else:
                        associate_member_line_ids = member_line_obj.search(cr, uid
                          , [ ('account_invoice_line', '=', line.id)
                            , ('partner', '!=', line.invoice_id.partner_id.id)
                            ]
                          , context=context)
                        member_line_obj.unlink(cr, uid, associate_member_line_ids, context=context)

                    #Define member ident if it's necessary
                    partners = [line.invoice_id.partner_id]
                    if line.invoice_id.partner_id.associate_members:
                        partners.extend(line.invoice_id.partner_id.associate_members)
                    for i in partners:
                        if not i.member_ident:
                            mbr_id = self.pool.get('ir.sequence').get(cr, uid, 'member_ident')
                            self.pool.get('res.partner').write(cr, uid, i.id, {'member_ident': mbr_id})
                if line.product_id and not line.product_id.membership and ml_ids:
                    # Product line has changed to a non membership product
                    member_line_obj.unlink(cr, uid, ml_ids, context=context)
        return res

    def create(self, cr, uid, vals, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res = super(account_invoice_line, self).create(cr, uid, vals, context=context)
        line = self.browse(cr, uid, res, context=context)

        if line.invoice_id.type == 'out_invoice' and line.product_id and line.product_id.membership:
            member_line_id = member_line_obj.search(cr, uid
              , [('partner'             , '=', line.invoice_id.partner_id.id)
                ,('account_invoice_line', '=', line.id)]
              , limit=1
              , context=context)
            for member_line in member_line_obj.browse(cr, uid, member_line_id, context=context):
              # NOTE: get member_line created in membership.account_invoice_line.create
                date_from = member_line.date_from
                date_to   = member_line.date_to
                if line.product_id.membership_date2date:
                    date_from = context['date_from'] and datetime.strptime(context['date_from'], "%Y-%m-%d") or date.today()
                    date_to   = date_from + relativedelta(months = +12) # TODO: parameterize this delta?
                    date_from = date_from.strftime("%Y-%m-%d")
                    date_to   = date_to  .strftime("%Y-%m-%d")
                member_line_obj.write(cr, uid, member_line.id
                  , {'date_from': date_from
                    ,'date_to'  : date_to
                    }
                  , context=context)
            partners = [line.invoice_id.partner_id]
            if line.product_id.membership_grouped and line.invoice_id.partner_id.associate_members:
                partners.extend(line.invoice_id.partner_id.associate_members)
                #Adding membership lines just for associate partners
                for associate_member in line.invoice_id.partner_id.associate_members:
                    member_line_obj.create(cr, uid, {
                        'partner': associate_member.id,
                        'membership_id': line.product_id.id,
                        'member_price': line.price_unit,
                        'date': time.strftime('%Y-%m-%d'),
                        'date_from': date_from,
                        'date_to': date_to,
                        'account_invoice_line': line.id,
                    }, context=context)
            #Define member ident if it's necessary
            for i in partners:
                if not i.member_ident:
                    mbr_id = self.pool.get('ir.sequence').get(cr, uid, 'member_ident')
                    self.pool.get('res.partner').write(cr, uid, i.id, {'member_ident': mbr_id})
        return res

account_invoice_line()


class pos_make_payment(osv.osv_memory):
    _inherit = 'pos.make.payment'

    def check(self, cr, uid, ids, context=None):
        """
        Auto invoice orders. Temporarly solution before make partner members just with orders, without invoice.
        """
        res = super(pos_make_payment, self).check(cr, uid, ids, context=context)

        order_obj = self.pool.get('pos.order')
        order_obj.action_invoice(cr, uid, ids, context=context)

        return res

pos_make_payment()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
